@@ -1,13 +1,16 @@
 from typing import Callable, List, Optional
+from contextlib import suppress
+
+from loguru import logger
 
 from wizwalker.memory.memory_object import DynamicMemoryObject, PropertyClass
 from .enums import WindowFlags, WindowStyle
 from .spell import DynamicGraphicalSpell
 from .combat_participant import DynamicCombatParticipant
-
-from wizwalker import Rectangle
+from wizwalker import AddressOutOfRange, MemoryReadError, Rectangle, utils
 
 
+# TODO: Window.click
 class Window(PropertyClass):
     async def read_base_address(self) -> int:
         raise NotImplementedError()
@@ -17,7 +20,7 @@ class Window(PropertyClass):
             f"{'-' * depth} [{await self.name()}] {await self.maybe_read_type_name()}"
         )
 
-        for child in await self.children():
+        for child in await utils.wait_for_non_error(self.children):
             await child.debug_print_ui_tree(depth + 1)
 
     async def debug_paint(self):
@@ -48,11 +51,12 @@ class Window(PropertyClass):
         return await self.get_windows_with_predicate(_pred)
 
     async def _recursive_get_windows_by_predicate(self, predicate, windows):
-        for child in await self.children():
-            if await predicate(child):
-                windows.append(child)
+        with suppress(ValueError, MemoryReadError, AddressOutOfRange):
+            for child in await self.children():
+                if await predicate(child):
+                    windows.append(child)
 
-            await child._recursive_get_windows_by_predicate(predicate, windows)
+                await child._recursive_get_windows_by_predicate(predicate, windows)
 
     async def get_windows_with_predicate(
         self, predicate: Callable
@@ -71,7 +75,7 @@ class Window(PropertyClass):
         # check our own children
         try:
             children = await self.children()
-        except ValueError:
+        except (ValueError, MemoryReadError, AddressOutOfRange):
             children = []
 
         for child in children:
@@ -100,11 +104,17 @@ class Window(PropertyClass):
 
         raise ValueError(f"No child named {name}")
 
+    async def is_visible(self):
+        return WindowFlags.visible in await self.flags()
+
     # This is here because checking in .children slows down window filtering majorly
-    async def maybe_graphical_spell(self) -> Optional[DynamicGraphicalSpell]:
-        type_name = await self.maybe_read_type_name()
-        if type_name != "SpellCheckBox":
-            raise ValueError(f"This object is a {type_name} not a SpellCheckBox.")
+    async def maybe_graphical_spell(
+        self, *, check_type: bool = False
+    ) -> Optional[DynamicGraphicalSpell]:
+        if check_type:
+            type_name = await self.maybe_read_type_name()
+            if type_name != "SpellCheckBox":
+                raise ValueError(f"This object is a {type_name} not a SpellCheckBox.")
 
         addr = await self.read_value_from_offset(952, "long long")
 
@@ -115,20 +125,24 @@ class Window(PropertyClass):
 
     # see maybe_graphical_spell
     # note: not defined
-    async def maybe_spell_grayed(self) -> bool:
-        type_name = await self.maybe_read_type_name()
-        if type_name != "SpellCheckBox":
-            raise ValueError(f"This object is a {type_name} not a SpellCheckBox")
+    async def maybe_spell_grayed(self, *, check_type: bool = False) -> bool:
+        if check_type:
+            type_name = await self.maybe_read_type_name()
+            if type_name != "SpellCheckBox":
+                raise ValueError(f"This object is a {type_name} not a SpellCheckBox")
 
         return await self.read_value_from_offset(1024, "bool")
 
     # See maybe_graphical_spell
-    async def maybe_combat_participant(self) -> Optional[DynamicCombatParticipant]:
-        type_name = await self.maybe_read_type_name()
-        if type_name != "CombatantDataControl":
-            raise ValueError(
-                f"This object is a {type_name} not a CombatantDataControl."
-            )
+    async def maybe_combat_participant(
+        self, *, check_type: bool = False
+    ) -> Optional[DynamicCombatParticipant]:
+        if check_type:
+            type_name = await self.maybe_read_type_name()
+            if type_name != "CombatantDataControl":
+                raise ValueError(
+                    f"This object is a {type_name} not a CombatantDataControl."
+                )
 
         addr = await self.read_value_from_offset(1592, "long long")
 
@@ -138,17 +152,16 @@ class Window(PropertyClass):
         return DynamicCombatParticipant(self.hook_handler, addr)
 
     # See maybe_graphical_spell
-    async def maybe_text(self) -> str:
+    async def maybe_text(self, *, check_type: bool = False) -> str:
         # TODO: see if all types with .text have Control prefix
         #  and if so check that they have it
         return await self.read_wide_string_from_offset(584)
 
-    # TODO: add back
-    # async def write_maybe_text(self, text: str):
-    #     """
-    #     Writing to this when there isn't actually a .text could crash
-    #     """
-    #     await self.write_string_to_offset(584, text)
+    async def write_maybe_text(self, text: str):
+        """
+        Writing to this when there isn't actually a .text could crash
+        """
+        await self.write_wide_string_to_offset(584, text)
 
     async def name(self) -> str:
         return await self.read_string_from_offset(80)
@@ -157,11 +170,19 @@ class Window(PropertyClass):
         await self.write_string_to_offset(80, name)
 
     async def children(self) -> List["DynamicWindow"]:
-        pointers = await self.read_shared_vector(112)
+        try:
+            pointers = await self.read_shared_vector(112)
+        except (ValueError, MemoryReadError):
+            logger.error("Issue while reading children vector raised to upper level")
+            return []
 
         windows = []
         for addr in pointers:
-            windows.append(DynamicWindow(self.hook_handler, addr))
+            if addr == 0:
+                logger.error("0 address while reading children")
+
+            else:
+                windows.append(DynamicWindow(self.hook_handler, addr))
 
         return windows
 
